/* *****************************************************************************
 * type/cas.c -- get the CAS type and data types out of raw identification data.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/* ************************************************************************** */
/*  Local types                                                               */
/* ************************************************************************** */
/* Correspondance type */
struct type_corresp {
	/* identification */
	const char *datatype;

	/* libg1m MCS file type, flags */
	g1m_mcstype_t type;
	unsigned int flags;
	g1m_pictureformat_t picformat;
};

/* ************************************************************************** */
/*  Correspondances                                                           */
/* ************************************************************************** */
/* All correspondances found by Tom Wheeley and Tom Lynn. Remarks:
 * - Correspondances with a NULL data type means the data type isn't to be
 *   read. */

#define UNIMPLEMENTED(S) {S, 0, 0, 0}
#define TTERM {NULL, 0, 0, 0}
static struct type_corresp cas_groups[] = {
	/* implemented */
	{"LT", g1m_mcstype_list,     0, 0},
	{"MT", g1m_mcstype_matrix,   0, 0},
	{"P1", g1m_mcstype_program,  0, 0},
	{"PG", g1m_mcstype_program,  0, 0},
	{"PZ", g1m_mcstype_program,  g1m_mcsflag_multiple, 0},
	{"VM", g1m_mcstype_variable, 0, 0},
	{"DC", g1m_mcstype_capture,  0, g1m_pictureformat_4bit_color},
	{"DD", g1m_mcstype_capture,  0, g1m_pictureformat_4bit_mono},

	/* not implemented yet */
	UNIMPLEMENTED("AA"), // dynamic graph functions
	UNIMPLEMENTED("AD"), // variable memory
	UNIMPLEMENTED("AL"), // all
	UNIMPLEMENTED("AM"), // alpha variable memory
	UNIMPLEMENTED("BU"), // backup
	UNIMPLEMENTED("DM"), // defined memory
	UNIMPLEMENTED("EN"), // one editor file
	UNIMPLEMENTED("FN"), // set of editor files
	UNIMPLEMENTED("FT"), // ??? (cafix)
	UNIMPLEMENTED("F1"), // one function memory
	UNIMPLEMENTED("F6"), // set of function memories
	UNIMPLEMENTED("GA"), // set of graph functions
	UNIMPLEMENTED("GF"), // graph zoom factor (graph function?)
	UNIMPLEMENTED("GM"), // gmem (cafix)
	UNIMPLEMENTED("GR"), // graph range
	UNIMPLEMENTED("GT"), // function table
	UNIMPLEMENTED("MA"), // set of matrices
	UNIMPLEMENTED("PC"), // picture from 9xxx (cafix)
	UNIMPLEMENTED("PD"), // polynomial equations
	UNIMPLEMENTED("RF"), // ??? (cafix)
	UNIMPLEMENTED("RR"), // ??? (cafix)
	UNIMPLEMENTED("RT"), // recursion table
	UNIMPLEMENTED("SD"), // simultaneous equations
	UNIMPLEMENTED("SE"), // equation (cafix)
	UNIMPLEMENTED("SR"), // linear regression data
	UNIMPLEMENTED("SS"), // standard deviation data
	UNIMPLEMENTED("TR"), // ??? (cafix)
	UNIMPLEMENTED("WD"), // window data (cafix)

	/* terminating entry */
	TTERM
};

/* ************************************************************************** */
/*  Main functions                                                            */
/* ************************************************************************** */
/**
 *	g1m_maketype_cas:
 *	Get the libg1m MCS type from raw CAS identification data.
 *
 *	@arg	head		the head to fill.
 *	@arg	datatype	the data type string.
 *	@return				the error (if any).
 */

int g1m_maketype_cas(g1m_mcshead_t *head, const char *datatype)
{
	/* copy information */
	memset(head, 0, sizeof(g1m_mcshead_t));
	head->flags |= g1m_mcsinfo_cas;
	memcpy(head->_datatype, datatype, 2);
	head->_datatype[2] = 0;

	/* look for correspondance */
	struct type_corresp *c = cas_groups - 1;
	while ((++c)->datatype) {
		if (c->datatype && strncmp(datatype, c->datatype, 4))
			continue;
		break;
	}
	if (!c->type) goto notfound;

	/* fill in info and return */
	head->type = c->type;
	head->_picformat = c->picformat;
	head->flags |= c->flags;
	return (0);

notfound:
	log_info("Type with '%.4s' data string was not implemented or not "
		"recognized.", datatype);
	head->type = 0;
	return (1);
}
